Package: RTMB
Type: Package
Title: 'R' Bindings for 'TMB'
Version: 1.8
Date: 2025-10-13
Authors@R: person("Kasper", "Kristensen", role = c("aut","cre"),
                  email = "kaskr@dtu.dk",
                  comment = c(ORCID="0000-0003-3425-3762"))
Author: Kasper Kristensen [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3425-3762>)
Maintainer: Kasper Kristensen <kaskr@dtu.dk>
Description: Native 'R' interface to 'TMB' (Template Model Builder) so models can be written entirely in 'R' rather than 'C++'. Automatic differentiation, to any order, is available for a rich subset of 'R' features, including linear algebra for dense and sparse matrices, complex arithmetic, Fast Fourier Transform, probability distributions and special functions. 'RTMB' provides easy access to model fitting and validation following the principles of Kristensen, K., Nielsen, A., Berg, C. W., Skaug, H., & Bell, B. M. (2016) <DOI:10.18637/jss.v070.i05> and Thygesen, U.H., Albertsen, C.M., Berg, C.W. et al. (2017) <DOI:10.1007/s10651-017-0372-4>.
License: GPL (>= 2)
Imports: Rcpp (>= 1.0.9), Matrix, methods, TMB (>= 1.9.7), MASS
LinkingTo: Rcpp, TMB, RcppEigen
VignetteBuilder: knitr, rmarkdown
Suggests: knitr, rmarkdown, igraph, tinytest, numDeriv
URL: https://github.com/kaskr/RTMB
BugReports: https://github.com/kaskr/RTMB/issues
NeedsCompilation: yes
Encoding: UTF-8
Packaged: 2025-10-13 17:30:11 UTC; kaskr
Repository: CRAN
Date/Publication: 2025-10-14 05:10:56 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2025-10-14 07:04:38 UTC; unix
Archs: RTMB.so.dSYM
